/**
 * Sigma.js WebGL Renderer Edge Program
 * =====================================
 *
 * Program rendering edges as thick lines using four points translated
 * orthogonally from the source & target's centers by half thickness.
 *
 * Rendering two triangles by using only four points is made possible through
 * the use of indices.
 *
 * This method should be faster than the 6 points / 2 triangles approach and
 * should handle thickness better than with gl.LINES.
 *
 * This version of the shader balances geometry computation evenly between
 * the CPU & GPU (normals are computed on the CPU side).
 */
import Program from "./program";
export default class EdgeProgram extends Program {
    IndicesArray: Uint32ArrayConstructor | Uint16ArrayConstructor;
    indicesArray: Uint32Array | Uint16Array;
    indicesBuffer: WebGLBuffer;
    indicesType: any;
    Locations: GLint;
    positionLocation: GLint;
    normalLocation: GLint;
    thicknessLocation: GLint;
    colorLocation: GLint;
    resolutionLocation: WebGLUniformLocation;
    ratioLocation: WebGLUniformLocation;
    matrixLocation: WebGLUniformLocation;
    scaleLocation: WebGLUniformLocation;
    canUse32BitsIndices: boolean;
    constructor(gl: WebGLRenderingContext);
    bind(): void;
    allocate(capacity: any): void;
    process(sourceData: any, targetData: any, data: any, offset: any): void;
    computeIndices(): void;
    bufferData(): void;
    render(params: any): void;
}
